package de.szut.casino.slots;

import de.szut.casino.shared.service.BalanceService;
import de.szut.casino.user.UserEntity;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Random;

@Service
public class SlotService {
    private final int REEL_LENGTH = 60;
    private static final int SEVEN_COUNT = 4;
    private static final int BAR_COUNT = 20;
    private static final int CHERRY_COUNT = 36;

    private final List<Symbol> FIRST_REEL = new ArrayList<>(REEL_LENGTH);
    private final List<Symbol> SECOND_REEL = new ArrayList<>(REEL_LENGTH);
    private final List<Symbol> THIRD_REEL = new ArrayList<>(REEL_LENGTH);

    private final BalanceService balanceService;

    public SlotService(BalanceService balanceService) {
        this.balanceService = balanceService;

        initReels();
    }

    public SpinResult handleSpinResult(
            BigDecimal betAmount,
            UserEntity user
            ) {
        Random random = new Random();

        int index1 = random.nextInt(REEL_LENGTH);
        int index2 = random.nextInt(REEL_LENGTH);
        int index3 = random.nextInt(REEL_LENGTH);

        Symbol symbol1 = FIRST_REEL.get(index1);
        Symbol symbol2 = SECOND_REEL.get(index2);
        Symbol symbol3 = THIRD_REEL.get(index3);

        boolean isWin = symbol1.equals(symbol2) && symbol1.equals(symbol3);

        SpinResult spinResult = new SpinResult();

        if (isWin) {
            BigDecimal winAmount = betAmount.multiply(symbol1.getPayoutMultiplier());
            this.balanceService.addFunds(user, winAmount);

            spinResult.setStatus("win");
            spinResult.setAmount(winAmount);
            spinResult.setWin(true);
        } else {
            this.balanceService.subtractFunds(user, betAmount);

            spinResult.setStatus("lose");
            spinResult.setAmount(betAmount);
            spinResult.setWin(false);
        }

        buildResultMatrix(spinResult, index1, index2, index3);

        return spinResult;
    }

    public void buildResultMatrix(SpinResult spinResult, int index1, int index2, int index3) {
        List<List<Symbol>> resultMatrix = new ArrayList<>(3);
        for (int i = 0; i < 3; i++) {
            resultMatrix.add(new ArrayList<>(3));
        }

        resultMatrix.get(0).add(getSymbolAt(FIRST_REEL, index1 - 1));
        resultMatrix.get(1).add(getSymbolAt(FIRST_REEL, index1));
        resultMatrix.get(2).add(getSymbolAt(FIRST_REEL, index1 + 1));

        resultMatrix.get(0).add(getSymbolAt(SECOND_REEL, index2 - 1));
        resultMatrix.get(1).add(getSymbolAt(SECOND_REEL, index2));
        resultMatrix.get(2).add(getSymbolAt(SECOND_REEL, index2 + 1));

        resultMatrix.get(0).add(getSymbolAt(THIRD_REEL, index3 - 1));
        resultMatrix.get(1).add(getSymbolAt(THIRD_REEL, index3));
        resultMatrix.get(2).add(getSymbolAt(THIRD_REEL, index3 + 1));

        spinResult.setResultMatrix(resultMatrix);
    }

    private void initReels() {
        List<Symbol> reelStrip = new ArrayList<>(REEL_LENGTH);

        for (int i = 0; i < CHERRY_COUNT; i++) {
            Symbol CHERRY = new Symbol("cherry", new BigDecimal(5));
            reelStrip.add(CHERRY);
        }

        for (int i = 0; i < BAR_COUNT; i++) {
            Symbol BAR = new Symbol("bar", new BigDecimal(10));
            reelStrip.add(BAR);
        }

        for (int i = 0; i < SEVEN_COUNT; i++) {
            Symbol SEVEN = new Symbol("seven", new BigDecimal(85));
            reelStrip.add(SEVEN);
        }

        FIRST_REEL.addAll(reelStrip);
        SECOND_REEL.addAll(reelStrip);
        THIRD_REEL.addAll(reelStrip);

        Collections.shuffle(FIRST_REEL);
        Collections.shuffle(SECOND_REEL);
        Collections.shuffle(THIRD_REEL);
    }

    private Symbol getSymbolAt(List<Symbol> reel, int index) {
        int effectiveIndex = index % REEL_LENGTH;

        if (effectiveIndex < 0) {
            effectiveIndex += REEL_LENGTH;
        }

        return reel.get(effectiveIndex);
    }
}
